@@ -8,6 +8,7 @@ import math
 import json
 import random
 import time
+from enum import Enum
 
 import requests
 from .excs import QQIOError
@@ -16,6 +17,14 @@ logger = logging.getLogger(__name__)
 
 api_base_url = 'http://c.y.qq.com'
 
+class CodeShouldBe200(QQIOError):
+    def __init__(self, data):
+        self._code = data['code']
+        self._data = data
+
+    def __str__(self):
+        return f'json code field should be 200, got {self._code}. data: {self._data}'
+
 
 def djb2(string):
     ''' Hash a word using the djb2 algorithm with the specified base. '''
@@ -286,6 +295,34 @@ class API(object):
         resp = requests.get(url, params=params)
         return resp.json()['data']
 
+    def playlist_remove_songs(self, playlist_id, song_id_list):
+        payload = {
+            'req_0': {
+                'method': 'DelSonglist',
+                'module': 'music.musicasset.PlaylistDetailWrite',
+                'param': {
+                    'dirId': playlist_id,  # int
+                    'v_songInfo': [{'songId': int(song_id), 'songType': 0} for song_id in song_id_list]
+                }
+            }
+        }
+        js = self.rpc(payload)
+        return js['req_0']['code'] == 0
+
+    def playlist_add_songs(self, playlist_id, song_id_list):
+        payload = {
+            'req_0': {
+                'method': 'AddSonglist',
+                'module': 'music.musicasset.PlaylistDetailWrite',
+                'param': {
+                    'dirId': int(playlist_id),  # int
+                    'v_songInfo': [{'songId': int(song_id), 'songType': 0} for song_id in song_id_list]
+                }
+            }
+        }
+        js = self.rpc(payload)
+        return js['req_0']['code'] == 0
+
     def playlist_detail(self, pid, offset=0, limit=50):
         url = api_base_url + '/qzone/fcg-bin/fcg_ucc_getcdinfo_byids_cp.fcg'
         params = {
@@ -318,6 +355,7 @@ class API(object):
         resp = requests.get(url, params=params, headers=self._headers,
                             cookies=self._cookies, timeout=self._timeout)
         js = resp.json()
+        logger.debug(f"user detail response: {js}")
         if js['code'] != 0:
             raise CodeShouldBe0(js)
         return js['data']
@@ -419,6 +457,13 @@ class API(object):
         }
         js = self.rpc(data)
         return js['req_0']['data']
+    
+    def get_comment(self, comment_id):
+        url = api_base_url + f'/base/fcgi-bin/fcg_global_comment_h5.fcg?biztype=1&cmd=8&topid={comment_id}&pagenum=0&pagesize=25'
+        res_data = requests.get(url, headers=self._headers)
+        if res_data.status_code == 200:
+            return res_data.json()
+        raise CodeShouldBe200(res_data)
 
     def get_lyric_by_songmid(self, songmid):
         url = api_base_url + '/lyric/fcgi-bin/fcg_query_lyric_new.fcg'
@@ -437,6 +482,7 @@ class API(object):
     def rpc(self, payload):
         if 'comm' not in payload:
             payload['comm'] = self.get_common_params()
+        logger.debug(f"rpc payload: {payload}")
         data_str = json.dumps(payload, ensure_ascii=False)
         params = {
             '_': int(round(time.time() * 1000)),
@@ -447,6 +493,7 @@ class API(object):
         resp = requests.get(url, params=params, headers=self._headers,
                             cookies=self._cookies, timeout=self._timeout)
         js = resp.json()
+        logger.debug(f"rpc response json: {js}")
         CodeShouldBe0.check(js)
         return js
 
@@ -717,5 +764,85 @@ class API(object):
             return 'http://isure.stream.qqmusic.qq.com/{}'.format(midurlinfo[0]['purl'])
         return ''
 
+    class DislikeListType(Enum):
+        singer = 2
+        song = 3
+        _style_unsupported = 4 # TODO: 这是什么？似乎是不喜欢的风格列表，不确定，暂时不支持
+
+    def get_dislike_list(self, page=1, type_=DislikeListType.song, last_id=0):
+        payload = {
+            "req_0": {
+                "module": "music.feedback.FeedbackBlack",
+                "method": "GetDislikeList",
+                "param": {
+                    "Cmd": type_.value,
+                    "Page": page,
+                    "SongLastid": last_id if type_ == API.DislikeListType.song else 0,
+                    "SingersLastid": (
+                        last_id if type_ == API.DislikeListType.singer else 0
+                    ),
+                },
+            },
+        }
+        js = self.rpc(payload)
+        if type_ == API.DislikeListType.song:
+            return js["req_0"]["data"]["Songs"]
+        elif type_ == API.DislikeListType.singer:
+            return js["req_0"]["data"]["Singers"]
+        else:
+            raise QQIOError(f"Unknown dislike list type: {type_}")
+
+    def add_to_dislike_list(self, items, type_=DislikeListType.song):
+        req_param = {
+            "Singers": [],
+            "Songs": [],
+            "Styles": [],
+            "OnlyAdd": 1,
+        }
+        if type_ == API.DislikeListType.song:
+            req_param["Songs"] = items
+        elif type_ == API.DislikeListType.singer:
+            req_param["Singers"] = items
+        else:
+            raise QQIOError(f"Unknown dislike list type: {type_}")
+
+        payload = {
+             "req_0": {
+                 "module": "music.feedback.FeedbackBlack",
+                 "method": "AddDislike",
+                 "param": req_param,
+             },
+        }
+        js = self.rpc(payload)
+        # Response example, {'code': 0, 'data': {'Retcode': 0, 'Msg': '', 'Token': ''}}
+        CodeShouldBe0.check(js['req_0'])
+        return js['req_0']['data']
+
+    def remove_from_dislike_list(self, items, type_=DislikeListType.song):
+        req_param = {
+            "Singers": [],
+            "Songs": [],
+            "Styles": [],
+            "OnlyAdd": 0,
+        }
+        if type_ == API.DislikeListType.song:
+            req_param["Songs"] = items
+        elif type_ == API.DislikeListType.singer:
+            req_param["Singers"] = items
+        else:
+            raise QQIOError(f"Unknown dislike list type: {type_}")
+
+        payload = {
+            "req_0": {
+                "module": "music.feedback.FeedbackBlack",
+                "method": "CancelDislike",
+                "param": req_param,
+            },
+        }
+        js = self.rpc(payload)
+        CodeShouldBe0.check(js)
+        CodeShouldBe0.check(js['req_0'])
+        return js['req_0']['data']
+
 
 api = API()
